@@ -20,14 +20,41 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-__version__ = '1.3.0'
+__version__ = '2.0.0'
 __author__ = 'Raphaël Barrois <raphael.barrois+fboy@polytechnique.org>'
 
 from .base import (
     Factory,
+    BaseDictFactory,
+    DictFactory,
+    BaseListFactory,
+    ListFactory,
+    MogoFactory,
     StubFactory,
     DjangoModelFactory,
 
+    BUILD_STRATEGY,
+    CREATE_STRATEGY,
+    STUB_STRATEGY,
+    use_strategy,
+)
+
+from .declarations import (
+    LazyAttribute,
+    Iterator,
+    Sequence,
+    LazyAttributeSequence,
+    SelfAttribute,
+    ContainerAttribute,
+    SubFactory,
+    Dict,
+    List,
+    PostGeneration,
+    PostGenerationMethodCall,
+    RelatedFactory,
+)
+
+from .helpers import (
     build,
     create,
     stub,
@@ -41,33 +68,8 @@ from .base import (
     generate_batch,
     simple_generate_batch,
 
-    BUILD_STRATEGY,
-    CREATE_STRATEGY,
-    STUB_STRATEGY,
-    use_strategy,
-
-    DJANGO_CREATION,
-    NAIVE_BUILD,
-    MOGO_BUILD,
-)
-
-from .declarations import (
-    LazyAttribute,
-    Iterator,
-    InfiniteIterator,
-    Sequence,
-    LazyAttributeSequence,
-    SelfAttribute,
-    ContainerAttribute,
-    SubFactory,
-    CircularSubFactory,
-    PostGeneration,
-    PostGenerationMethodCall,
-    RelatedFactory,
-
     lazy_attribute,
     iterator,
-    infinite_iterator,
     sequence,
     lazy_attribute_sequence,
     container_attribute,
