@@ -20,31 +20,13 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-import re
-import sys
-import warnings
-
-from factory import containers
+from . import containers
 
 # Strategies
 BUILD_STRATEGY = 'build'
 CREATE_STRATEGY = 'create'
 STUB_STRATEGY = 'stub'
 
-# Creation functions. Deprecated.
-# Override Factory._create instead.
-def DJANGO_CREATION(class_to_create, **kwargs):
-    warnings.warn(
-        "Factories defaulting to Django's Foo.objects.create() is deprecated, "
-        "and will be removed in the future. Please inherit from "
-        "factory.DjangoModelFactory instead.", PendingDeprecationWarning, 6)
-    return class_to_create.objects.create(**kwargs)
-
-# Building functions. Deprecated.
-# Override Factory._build instead.
-NAIVE_BUILD = lambda class_to_build, **kwargs: class_to_build(**kwargs)
-MOGO_BUILD = lambda class_to_build, **kwargs: class_to_build.new(**kwargs)
-
 
 # Special declarations
 FACTORY_CLASS_DECLARATION = 'FACTORY_FOR'
@@ -55,32 +37,120 @@ CLASS_ATTRIBUTE_POSTGEN_DECLARATIONS = '_postgen_declarations'
 CLASS_ATTRIBUTE_ASSOCIATED_CLASS = '_associated_class'
 
 
+class FactoryError(Exception):
+    """Any exception raised by factory_boy."""
+
+
+class AssociatedClassError(FactoryError):
+    """Exception for Factory subclasses lacking FACTORY_FOR."""
+
+
+class UnknownStrategy(FactoryError):
+    """Raised when a factory uses an unknown strategy."""
+
+
+class UnsupportedStrategy(FactoryError):
+    """Raised when trying to use a strategy on an incompatible Factory."""
+
+
 # Factory metaclasses
 
 def get_factory_bases(bases):
-    """Retrieve all BaseFactoryMetaClass-derived bases from a list."""
-    return [b for b in bases if isinstance(b, BaseFactoryMetaClass)]
+    """Retrieve all FactoryMetaClass-derived bases from a list."""
+    return [b for b in bases if issubclass(b, BaseFactory)]
 
 
-class BaseFactoryMetaClass(type):
+class FactoryMetaClass(type):
     """Factory metaclass for handling ordered declarations."""
 
     def __call__(cls, **kwargs):
-        """Override the default Factory() syntax to call the default build strategy.
+        """Override the default Factory() syntax to call the default strategy.
 
         Returns an instance of the associated class.
         """
 
-        if cls.default_strategy == BUILD_STRATEGY:
+        if cls.FACTORY_STRATEGY == BUILD_STRATEGY:
             return cls.build(**kwargs)
-        elif cls.default_strategy == CREATE_STRATEGY:
+        elif cls.FACTORY_STRATEGY == CREATE_STRATEGY:
             return cls.create(**kwargs)
-        elif cls.default_strategy == STUB_STRATEGY:
+        elif cls.FACTORY_STRATEGY == STUB_STRATEGY:
             return cls.stub(**kwargs)
         else:
-            raise BaseFactory.UnknownStrategy('Unknown default_strategy: {0}'.format(cls.default_strategy))
+            raise UnknownStrategy('Unknown FACTORY_STRATEGY: {0}'.format(
+                cls.FACTORY_STRATEGY))
 
-    def __new__(cls, class_name, bases, attrs, extra_attrs=None):
+    @classmethod
+    def _discover_associated_class(mcs, class_name, attrs, inherited=None):
+        """Try to find the class associated with this factory.
+
+        In order, the following tests will be performed:
+        - Lookup the FACTORY_CLASS_DECLARATION attribute
+        - If an inherited associated class was provided, use it.
+
+        Args:
+            class_name (str): the name of the factory class being created
+            attrs (dict): the dict of attributes from the factory class
+                definition
+            inherited (class): the optional associated class inherited from a
+                parent factory
+
+        Returns:
+            class: the class to associate with this factory
+
+        Raises:
+            AssociatedClassError: If we were unable to associate this factory
+                to a class.
+        """
+        if FACTORY_CLASS_DECLARATION in attrs:
+            return attrs[FACTORY_CLASS_DECLARATION]
+
+        # No specific associated class was given, and one was defined for our
+        # parent, use it.
+        if inherited is not None:
+            return inherited
+
+        raise AssociatedClassError(
+            "Could not determine the class associated with %s. "
+            "Use the FACTORY_FOR attribute to specify an associated class." %
+            class_name)
+
+    @classmethod
+    def _extract_declarations(mcs, bases, attributes):
+        """Extract declarations from a class definition.
+
+        Args:
+            bases (class list): parent Factory subclasses
+            attributes (dict): attributes declared in the class definition
+
+        Returns:
+            dict: the original attributes, where declarations have been moved to
+                _declarations and post-generation declarations to
+                _postgen_declarations.
+        """
+        declarations = containers.DeclarationDict()
+        postgen_declarations = containers.PostGenerationDeclarationDict()
+
+        # Add parent declarations in reverse order.
+        for base in reversed(bases):
+            # Import parent PostGenerationDeclaration
+            postgen_declarations.update_with_public(
+                getattr(base, CLASS_ATTRIBUTE_POSTGEN_DECLARATIONS, {}))
+            # Import all 'public' attributes (avoid those starting with _)
+            declarations.update_with_public(
+                    getattr(base, CLASS_ATTRIBUTE_DECLARATIONS, {}))
+
+        # Import attributes from the class definition
+        attributes = postgen_declarations.update_with_public(attributes)
+        # Store protected/private attributes in 'non_factory_attrs'.
+        attributes = declarations.update_with_public(attributes)
+
+        # Store the DeclarationDict in the attributes of the newly created class
+        attributes[CLASS_ATTRIBUTE_DECLARATIONS] = declarations
+        attributes[CLASS_ATTRIBUTE_POSTGEN_DECLARATIONS] = postgen_declarations
+
+        return attributes
+
+    def __new__(mcs, class_name, bases, attrs, extra_attrs=None):
         """Record attributes as a pattern for later instance construction.
 
         This is called when a new Factory subclass is defined; it will collect
@@ -98,164 +168,59 @@ class BaseFactoryMetaClass(type):
         Returns:
             A new class
         """
-
         parent_factories = get_factory_bases(bases)
         if not parent_factories:
-            # If this isn't a subclass of Factory, don't do anything special.
-            return super(BaseFactoryMetaClass, cls).__new__(cls, class_name, bases, attrs)
+            return super(FactoryMetaClass, mcs).__new__(
+                    mcs, class_name, bases, attrs)
 
-        declarations = containers.DeclarationDict()
-        postgen_declarations = containers.PostGenerationDeclarationDict()
+        is_abstract = attrs.pop('ABSTRACT_FACTORY', False)
+        extra_attrs = {}
 
-        # Add parent declarations in reverse order.
-        for base in reversed(parent_factories):
-            # Import parent PostGenerationDeclaration
-            postgen_declarations.update_with_public(
-                getattr(base, CLASS_ATTRIBUTE_POSTGEN_DECLARATIONS, {}))
-            # Import all 'public' attributes (avoid those starting with _)
-            declarations.update_with_public(getattr(base, CLASS_ATTRIBUTE_DECLARATIONS, {}))
+        if not is_abstract:
 
-        # Import attributes from the class definition
-        non_postgen_attrs = postgen_declarations.update_with_public(attrs)
-        # Store protected/private attributes in 'non_factory_attrs'.
-        non_factory_attrs = declarations.update_with_public(non_postgen_attrs)
+            base = parent_factories[0]
 
-        # Store the DeclarationDict in the attributes of the newly created class
-        non_factory_attrs[CLASS_ATTRIBUTE_DECLARATIONS] = declarations
-        non_factory_attrs[CLASS_ATTRIBUTE_POSTGEN_DECLARATIONS] = postgen_declarations
+            inherited_associated_class = getattr(base,
+                    CLASS_ATTRIBUTE_ASSOCIATED_CLASS, None)
+            associated_class = mcs._discover_associated_class(class_name, attrs,
+                    inherited_associated_class)
+
+            # If inheriting the factory from a parent, keep a link to it.
+            # This allows to use the sequence counters from the parents.
+            if associated_class == inherited_associated_class:
+                attrs['_base_factory'] = base
+
+            # The CLASS_ATTRIBUTE_ASSOCIATED_CLASS must *not* be taken into
+            # account when parsing the declared attributes of the new class.
+            extra_attrs = {CLASS_ATTRIBUTE_ASSOCIATED_CLASS: associated_class}
+
+        # Extract pre- and post-generation declarations
+        attributes = mcs._extract_declarations(parent_factories, attrs)
 
         # Add extra args if provided.
         if extra_attrs:
-            non_factory_attrs.update(extra_attrs)
+            attributes.update(extra_attrs)
 
-        return super(BaseFactoryMetaClass, cls).__new__(cls, class_name, bases, non_factory_attrs)
+        return super(FactoryMetaClass, mcs).__new__(
+                mcs, class_name, bases, attributes)
 
+    def __str__(cls):
+        return '<%s for %s>' % (cls.__name__,
+            getattr(cls, CLASS_ATTRIBUTE_ASSOCIATED_CLASS).__name__)
 
-class FactoryMetaClass(BaseFactoryMetaClass):
-    """Factory metaclass for handling class association and ordered declarations."""
-
-    ERROR_MESSAGE = """Could not determine what class this factory is for.
-    Use the {0} attribute to specify a class."""
-    ERROR_MESSAGE_AUTODISCOVERY = ERROR_MESSAGE + """
-    Also, autodiscovery failed using the name '{1}'
-    based on the Factory name '{2}' in {3}."""
-
-    @classmethod
-    def _discover_associated_class(cls, class_name, attrs, inherited=None):
-        """Try to find the class associated with this factory.
-
-        In order, the following tests will be performed:
-        - Lookup the FACTORY_CLASS_DECLARATION attribute
-        - If the newly created class is named 'FooBarFactory', look for a FooBar
-            class in its module
-        - If an inherited associated class was provided, use it.
-
-        Args:
-            class_name (str): the name of the factory class being created
-            attrs (dict): the dict of attributes from the factory class
-                definition
-            inherited (class): the optional associated class inherited from a
-                parent factory
-
-        Returns:
-            class: the class to associate with this factory
-
-        Raises:
-            AssociatedClassError: If we were unable to associate this factory
-                to a class.
-        """
-        own_associated_class = None
-        used_auto_discovery = False
-
-        if FACTORY_CLASS_DECLARATION in attrs:
-            return attrs[FACTORY_CLASS_DECLARATION]
-
-        # No specific associated class was given, and one was defined for our
-        # parent, use it.
-        if inherited is not None:
-            return inherited
-
-        if '__module__' in attrs:
-            factory_module = sys.modules[attrs['__module__']]
-            if class_name.endswith('Factory'):
-                # Try a module lookup
-                used_auto_discovery = True
-                associated_name = class_name[:-len('Factory')]
-                if associated_name and hasattr(factory_module, associated_name):
-                    warnings.warn(
-                        "Auto-discovery of associated class is deprecated, and "
-                        "will be removed in the future. Please set '%s = %s' "
-                        "in the %s class definition." % (
-                            FACTORY_CLASS_DECLARATION,
-                            associated_name,
-                            class_name,
-                        ), DeprecationWarning, 3)
-
-                    return getattr(factory_module, associated_name)
-
-        # Unable to guess a good option; return the inherited class.
-        # Unable to find an associated class; fail.
-        if used_auto_discovery:
-            raise Factory.AssociatedClassError(
-                FactoryMetaClass.ERROR_MESSAGE_AUTODISCOVERY.format(
-                    FACTORY_CLASS_DECLARATION,
-                    associated_name,
-                    class_name,
-                    factory_module,))
-        else:
-            raise Factory.AssociatedClassError(
-                FactoryMetaClass.ERROR_MESSAGE.format(
-                    FACTORY_CLASS_DECLARATION))
-
-    def __new__(cls, class_name, bases, attrs):
-        """Determine the associated class based on the factory class name. Record the associated class
-        for construction of an associated class instance at a later time."""
-
-        parent_factories = get_factory_bases(bases)
-        if not parent_factories or attrs.get('ABSTRACT_FACTORY', False):
-            # If this isn't a subclass of Factory, or specifically declared
-            # abstract, don't do anything special.
-            if 'ABSTRACT_FACTORY' in attrs:
-                attrs.pop('ABSTRACT_FACTORY')
-
-            return super(FactoryMetaClass, cls).__new__(cls, class_name, bases, attrs)
-
-        base = parent_factories[0]
-
-        inherited_associated_class = getattr(base,
-                CLASS_ATTRIBUTE_ASSOCIATED_CLASS, None)
-        associated_class = cls._discover_associated_class(class_name, attrs,
-                inherited_associated_class)
-
-        # If inheriting the factory from a parent, keep a link to it.
-        # This allows to use the sequence counters from the parents.
-        if associated_class == inherited_associated_class:
-            attrs['_base_factory'] = base
-
-        # The CLASS_ATTRIBUTE_ASSOCIATED_CLASS must *not* be taken into account
-        # when parsing the declared attributes of the new class.
-        extra_attrs = {CLASS_ATTRIBUTE_ASSOCIATED_CLASS: associated_class}
-
-        return super(FactoryMetaClass, cls).__new__(cls, class_name, bases, attrs, extra_attrs=extra_attrs)
-
-    def __str__(self):
-        return '<%s for %s>' % (self.__name__,
-            getattr(self, CLASS_ATTRIBUTE_ASSOCIATED_CLASS).__name__)
 
 # Factory base classes
 
 class BaseFactory(object):
     """Factory base support for sequences, attributes and stubs."""
 
-    class UnknownStrategy(RuntimeError):
-        pass
-
-    class UnsupportedStrategy(RuntimeError):
-        pass
+    # Backwards compatibility
+    UnknownStrategy = UnknownStrategy
+    UnsupportedStrategy = UnsupportedStrategy
 
     def __new__(cls, *args, **kwargs):
         """Would be called if trying to instantiate the class."""
-        raise RuntimeError('You cannot instantiate BaseFactory')
+        raise FactoryError('You cannot instantiate BaseFactory')
 
     # ID to use for the next 'declarations.Sequence' attribute.
     _next_sequence = None
@@ -265,9 +230,15 @@ class BaseFactory(object):
     # class.
     _base_factory = None
 
+    # Holds the target class, once resolved.
+    _associated_class = None
+
     # List of arguments that should be passed as *args instead of **kwargs
     FACTORY_ARG_PARAMETERS = ()
 
+    # List of attributes that should not be passed to the underlying class
+    FACTORY_HIDDEN_ARGS = ()
+
     @classmethod
     def _setup_next_sequence(cls):
         """Set up an initial sequence value for Sequence attributes.
@@ -311,7 +282,13 @@ class BaseFactory(object):
             applicable; the current list of computed attributes is available
             to the currently processed object.
         """
-        return containers.AttributeBuilder(cls, extra).build(create)
+        force_sequence = None
+        if extra:
+            force_sequence = extra.pop('__sequence', None)
+        return containers.AttributeBuilder(cls, extra).build(
+            create=create,
+            force_sequence=force_sequence,
+        )
 
     @classmethod
     def declarations(cls, extra_defs=None):
@@ -323,6 +300,74 @@ class BaseFactory(object):
         """
         return getattr(cls, CLASS_ATTRIBUTE_DECLARATIONS).copy(extra_defs)
 
+    @classmethod
+    def _adjust_kwargs(cls, **kwargs):
+        """Extension point for custom kwargs adjustment."""
+        return kwargs
+
+    @classmethod
+    def _prepare(cls, create, **kwargs):
+        """Prepare an object for this factory.
+
+        Args:
+            create: bool, whether to create or to build the object
+            **kwargs: arguments to pass to the creation function
+        """
+        target_class = getattr(cls, CLASS_ATTRIBUTE_ASSOCIATED_CLASS)
+        kwargs = cls._adjust_kwargs(**kwargs)
+
+        # Remove 'hidden' arguments.
+        for arg in cls.FACTORY_HIDDEN_ARGS:
+            del kwargs[arg]
+
+        # Extract *args from **kwargs
+        args = tuple(kwargs.pop(key) for key in cls.FACTORY_ARG_PARAMETERS)
+
+        if create:
+            return cls._create(target_class, *args, **kwargs)
+        else:
+            return cls._build(target_class, *args, **kwargs)
+
+    @classmethod
+    def _generate(cls, create, attrs):
+        """generate the object.
+
+        Args:
+            create (bool): whether to 'build' or 'create' the object
+            attrs (dict): attributes to use for generating the object
+        """
+        # Extract declarations used for post-generation
+        postgen_declarations = getattr(cls,
+                CLASS_ATTRIBUTE_POSTGEN_DECLARATIONS)
+        postgen_attributes = {}
+        for name, decl in sorted(postgen_declarations.items()):
+            postgen_attributes[name] = decl.extract(name, attrs)
+
+        # Generate the object
+        obj = cls._prepare(create, **attrs)
+
+        # Handle post-generation attributes
+        results = {}
+        for name, decl in sorted(postgen_declarations.items()):
+            extracted, extracted_kwargs = postgen_attributes[name]
+            results[name] = decl.call(obj, create, extracted,
+                    **extracted_kwargs)
+
+        cls._after_postgeneration(obj, create, results)
+
+        return obj
+
+    @classmethod
+    def _after_postgeneration(cls, obj, create, results=None):
+        """Hook called after post-generation declarations have been handled.
+
+        Args:
+            obj (object): the generated object
+            create (bool): whether the strategy was 'build' or 'create'
+            results (dict or None): result of post-generation declarations
+        """
+        pass
+
     @classmethod
     def _build(cls, target_class, *args, **kwargs):
         """Actually build an instance of the target_class.
@@ -356,7 +401,8 @@ class BaseFactory(object):
     @classmethod
     def build(cls, **kwargs):
         """Build an instance of the associated class, with overriden attrs."""
-        raise cls.UnsupportedStrategy()
+        attrs = cls.attributes(create=False, extra=kwargs)
+        return cls._generate(False, attrs)
 
     @classmethod
     def build_batch(cls, size, **kwargs):
@@ -373,7 +419,8 @@ class BaseFactory(object):
     @classmethod
     def create(cls, **kwargs):
         """Create an instance of the associated class, with overriden attrs."""
-        raise cls.UnsupportedStrategy()
+        attrs = cls.attributes(create=True, extra=kwargs)
+        return cls._generate(True, attrs)
 
     @classmethod
     def create_batch(cls, size, **kwargs):
@@ -478,192 +525,63 @@ class BaseFactory(object):
         return cls.generate_batch(strategy, size, **kwargs)
 
 
-class StubFactory(BaseFactory):
-    __metaclass__ = BaseFactoryMetaClass
-
-    default_strategy = STUB_STRATEGY
-
-
-class Factory(BaseFactory):
-    """Factory base with build and create support.
+Factory = FactoryMetaClass('Factory', (BaseFactory,), {
+    'ABSTRACT_FACTORY': True,
+    'FACTORY_STRATEGY': CREATE_STRATEGY,
+    '__doc__': """Factory base with build and create support.
 
     This class has the ability to support multiple ORMs by using custom creation
     functions.
-    """
-    __metaclass__ = FactoryMetaClass
+    """,
+    })
 
-    default_strategy = CREATE_STRATEGY
 
-    class AssociatedClassError(RuntimeError):
-        pass
+# Backwards compatibility
+Factory.AssociatedClassError = AssociatedClassError  # pylint: disable=W0201
 
-    @classmethod
-    def set_creation_function(cls, creation_function):
-        """Set the creation function for this class.
 
-        Args:
-            creation_function (function): the new creation function. That
-                function should take one non-keyword argument, the 'class' for
-                which an instance will be created. The value of the various
-                fields are passed as keyword arguments.
-        """
-        warnings.warn(
-            "Use of factory.set_creation_function is deprecated, and will be "
-            "removed in the future. Please override Factory._create() instead.",
-            PendingDeprecationWarning, 2)
-        # Customizing 'create' strategy, using a tuple to keep the creation function
-        # from turning it into an instance method.
-        cls._creation_function = (creation_function,)
+class StubFactory(Factory):
 
-    @classmethod
-    def get_creation_function(cls):
-        """Retrieve the creation function for this class.
-
-        Returns:
-            function: A function that takes one parameter, the class for which
-                an instance will be created, and keyword arguments for the value
-                of the fields of the instance.
-        """
-        creation_function = getattr(cls, '_creation_function', ())
-        if creation_function and creation_function[0]:
-            return creation_function[0]
-        elif cls._create.__func__ == Factory._create.__func__ and \
-                hasattr(cls._associated_class, 'objects'):
-            # Backwards compatibility.
-            # Default creation_function and default _create() behavior.
-            # The best "Vanilla" _create detection algorithm I found is relying
-            # on actual method implementation (otherwise, make_factory isn't
-            # detected as 'default').
-            return DJANGO_CREATION
-
-    @classmethod
-    def set_building_function(cls, building_function):
-        """Set the building function for this class.
-
-        Args:
-            building_function (function): the new building function. That
-                function should take one non-keyword argument, the 'class' for
-                which an instance will be built. The value of the various
-                fields are passed as keyword arguments.
-        """
-        warnings.warn(
-            "Use of factory.set_building_function is deprecated, and will be "
-            "removed in the future. Please override Factory._build() instead.",
-            PendingDeprecationWarning, 2)
-        # Customizing 'build' strategy, using a tuple to keep the creation function
-        # from turning it into an instance method.
-        cls._building_function = (building_function,)
-
-    @classmethod
-    def get_building_function(cls):
-        """Retrieve the building function for this class.
-
-        Returns:
-            function: A function that takes one parameter, the class for which
-                an instance will be created, and keyword arguments for the value
-                of the fields of the instance.
-        """
-        building_function = getattr(cls, '_building_function', ())
-        if building_function and building_function[0]:
-            return building_function[0]
-
-    @classmethod
-    def _adjust_kwargs(cls, **kwargs):
-        """Extension point for custom kwargs adjustment."""
-        return kwargs
-
-    @classmethod
-    def _prepare(cls, create, **kwargs):
-        """Prepare an object for this factory.
-
-        Args:
-            create: bool, whether to create or to build the object
-            **kwargs: arguments to pass to the creation function
-        """
-        target_class = getattr(cls, CLASS_ATTRIBUTE_ASSOCIATED_CLASS)
-        kwargs = cls._adjust_kwargs(**kwargs)
-
-        # Extract *args from **kwargs
-        args = tuple(kwargs.pop(key) for key in cls.FACTORY_ARG_PARAMETERS)
-
-        if create:
-            # Backwards compatibility
-            creation_function = cls.get_creation_function()
-            if creation_function:
-                return creation_function(target_class, *args, **kwargs)
-            return cls._create(target_class, *args, **kwargs)
-        else:
-            # Backwards compatibility
-            building_function = cls.get_building_function()
-            if building_function:
-                return building_function(target_class, *args, **kwargs)
-            return cls._build(target_class, *args, **kwargs)
-
-    @classmethod
-    def _generate(cls, create, attrs):
-        """generate the object.
-
-        Args:
-            create (bool): whether to 'build' or 'create' the object
-            attrs (dict): attributes to use for generating the object
-        """
-        # Extract declarations used for post-generation
-        postgen_declarations = getattr(cls, CLASS_ATTRIBUTE_POSTGEN_DECLARATIONS)
-        postgen_attributes = {}
-        for name, decl in sorted(postgen_declarations.items()):
-            postgen_attributes[name] = decl.extract(name, attrs)
-
-        # Generate the object
-        obj = cls._prepare(create, **attrs)
-
-        # Handle post-generation attributes
-        results = {}
-        for name, decl in sorted(postgen_declarations.items()):
-            extracted, extracted_kwargs = postgen_attributes[name]
-            results[name] = decl.call(obj, create, extracted, **extracted_kwargs)
-
-        cls._after_postgeneration(obj, create, results)
-
-        return obj
-
-    @classmethod
-    def _after_postgeneration(cls, obj, create, results=None):
-        """Hook called after post-generation declarations have been handled.
-
-        Args:
-            obj (object): the generated object
-            create (bool): whether the strategy was 'build' or 'create'
-            results (dict or None): result of post-generation declarations
-        """
-        pass
+    FACTORY_STRATEGY = STUB_STRATEGY
+    FACTORY_FOR = containers.StubObject
 
     @classmethod
     def build(cls, **kwargs):
-        attrs = cls.attributes(create=False, extra=kwargs)
-        return cls._generate(False, attrs)
+        raise UnsupportedStrategy()
 
     @classmethod
     def create(cls, **kwargs):
-        attrs = cls.attributes(create=True, extra=kwargs)
-        return cls._generate(True, attrs)
+        raise UnsupportedStrategy()
 
 
 class DjangoModelFactory(Factory):
     """Factory for Django models.
 
-    This makes sure that the 'sequence' field of created objects is an unused id.
+    This makes sure that the 'sequence' field of created objects is a new id.
 
     Possible improvement: define a new 'attribute' type, AutoField, which would
     handle those for non-numerical primary keys.
     """
 
     ABSTRACT_FACTORY = True
+    FACTORY_DJANGO_GET_OR_CREATE = ()
+
+    @classmethod
+    def _get_manager(cls, target_class):
+        try:
+            return target_class._default_manager   # pylint: disable=W0212
+        except AttributeError:
+            return target_class.objects
 
     @classmethod
     def _setup_next_sequence(cls):
         """Compute the next available PK, based on the 'pk' database field."""
+
+        model = cls._associated_class  # pylint: disable=E1101
+        manager = cls._get_manager(model)
+
         try:
-            return 1 + cls._associated_class._default_manager.values_list('pk', flat=True
+            return 1 + manager.values_list('pk', flat=True
                 ).order_by('-pk')[0]
         except IndexError:
             return 1
@@ -671,7 +589,20 @@ class DjangoModelFactory(Factory):
     @classmethod
     def _create(cls, target_class, *args, **kwargs):
         """Create an instance of the model, and save it to the database."""
-        return target_class._default_manager.create(*args, **kwargs)
+        manager = cls._get_manager(target_class)
+
+        assert 'defaults' not in cls.FACTORY_DJANGO_GET_OR_CREATE, (
+            "'defaults' is a reserved keyword for get_or_create "
+            "(in %s.FACTORY_DJANGO_GET_OR_CREATE=%r)"
+            % (cls, cls.FACTORY_DJANGO_GET_OR_CREATE))
+
+        key_fields = {}
+        for field in cls.FACTORY_DJANGO_GET_OR_CREATE:
+            key_fields[field] = kwargs.pop(field)
+        key_fields['defaults'] = kwargs
+
+        obj, _created = manager.get_or_create(*args, **key_fields)
+        return obj
 
     @classmethod
     def _after_postgeneration(cls, obj, create, results=None):
@@ -690,64 +621,46 @@ class MogoFactory(Factory):
         return target_class.new(*args, **kwargs)
 
 
-def make_factory(klass, **kwargs):
-    """Create a new, simple factory for the given class."""
-    factory_name = '%sFactory' % klass.__name__
-    kwargs[FACTORY_CLASS_DECLARATION] = klass
-    factory_class = type(Factory).__new__(type(Factory), factory_name, (Factory,), kwargs)
-    factory_class.__name__ = '%sFactory' % klass.__name__
-    factory_class.__doc__ = 'Auto-generated factory for class %s' % klass
-    return factory_class
+class BaseDictFactory(Factory):
+    """Factory for dictionary-like classes."""
+    ABSTRACT_FACTORY = True
+
+    @classmethod
+    def _build(cls, target_class, *args, **kwargs):
+        if args:
+            raise ValueError(
+                "DictFactory %r does not support FACTORY_ARG_PARAMETERS.", cls)
+        return target_class(**kwargs)
+
+    @classmethod
+    def _create(cls, target_class, *args, **kwargs):
+        return cls._build(target_class, *args, **kwargs)
 
 
-def build(klass, **kwargs):
-    """Create a factory for the given class, and build an instance."""
-    return make_factory(klass, **kwargs).build()
+class DictFactory(BaseDictFactory):
+    FACTORY_FOR = dict
 
 
-def build_batch(klass, size, **kwargs):
-    """Create a factory for the given class, and build a batch of instances."""
-    return make_factory(klass, **kwargs).build_batch(size)
+class BaseListFactory(Factory):
+    """Factory for list-like classes."""
+    ABSTRACT_FACTORY = True
+
+    @classmethod
+    def _build(cls, target_class, *args, **kwargs):
+        if args:
+            raise ValueError(
+                "ListFactory %r does not support FACTORY_ARG_PARAMETERS.", cls)
+
+        values = [v for k, v in sorted(kwargs.items())]
+        return target_class(values)
+
+    @classmethod
+    def _create(cls, target_class, *args, **kwargs):
+        return cls._build(target_class, *args, **kwargs)
 
 
-def create(klass, **kwargs):
-    """Create a factory for the given class, and create an instance."""
-    return make_factory(klass, **kwargs).create()
-
-
-def create_batch(klass, size, **kwargs):
-    """Create a factory for the given class, and create a batch of instances."""
-    return make_factory(klass, **kwargs).create_batch(size)
-
-
-def stub(klass, **kwargs):
-    """Create a factory for the given class, and stub an instance."""
-    return make_factory(klass, **kwargs).stub()
-
-
-def stub_batch(klass, size, **kwargs):
-    """Create a factory for the given class, and stub a batch of instances."""
-    return make_factory(klass, **kwargs).stub_batch(size)
-
-
-def generate(klass, strategy, **kwargs):
-    """Create a factory for the given class, and generate an instance."""
-    return make_factory(klass, **kwargs).generate(strategy)
-
-
-def generate_batch(klass, strategy, size, **kwargs):
-    """Create a factory for the given class, and generate instances."""
-    return make_factory(klass, **kwargs).generate_batch(strategy, size)
-
-
-def simple_generate(klass, create, **kwargs):
-    """Create a factory for the given class, and simple_generate an instance."""
-    return make_factory(klass, **kwargs).simple_generate(create)
-
-
-def simple_generate_batch(klass, create, size, **kwargs):
-    """Create a factory for the given class, and simple_generate instances."""
-    return make_factory(klass, **kwargs).simple_generate_batch(create, size)
+class ListFactory(BaseListFactory):
+    FACTORY_FOR = list
 
 
 def use_strategy(new_strategy):
@@ -756,6 +669,6 @@ def use_strategy(new_strategy):
     This is an alternative to setting default_strategy in the class definition.
     """
     def wrapped_class(klass):
-        klass.default_strategy = new_strategy
+        klass.FACTORY_STRATEGY = new_strategy
         return klass
     return wrapped_class
