@@ -49,10 +49,17 @@ class FakeModel(object):
         return instance
 
     class FakeModelManager(object):
-        def create(self, **kwargs):
+        def get_or_create(self, **kwargs):
+            kwargs.update(kwargs.pop('defaults', {}))
             instance = FakeModel.create(**kwargs)
             instance.id = 2
-            return instance
+            return instance, True
+
+        def values_list(self, *args, **kwargs):
+            return self
+
+        def order_by(self, *args, **kwargs):
+            return [1]
 
     objects = FakeModelManager()
 
@@ -104,19 +111,33 @@ class SimpleBuildTestCase(unittest.TestCase):
             self.assertEqual(obj.three, 3)
             self.assertEqual(obj.four, None)
 
-    @tools.disable_warnings
     def test_create(self):
         obj = factory.create(FakeModel, foo='bar')
+        self.assertEqual(obj.id, None)
+        self.assertEqual(obj.foo, 'bar')
+
+    def test_create_custom_base(self):
+        obj = factory.create(FakeModel, foo='bar', FACTORY_CLASS=factory.DjangoModelFactory)
         self.assertEqual(obj.id, 2)
         self.assertEqual(obj.foo, 'bar')
 
-    @tools.disable_warnings
     def test_create_batch(self):
         objs = factory.create_batch(FakeModel, 4, foo='bar')
 
         self.assertEqual(4, len(objs))
         self.assertEqual(4, len(set(objs)))
 
+        for obj in objs:
+            self.assertEqual(obj.id, None)
+            self.assertEqual(obj.foo, 'bar')
+
+    def test_create_batch_custom_base(self):
+        objs = factory.create_batch(FakeModel, 4, foo='bar',
+                FACTORY_CLASS=factory.DjangoModelFactory)
+
+        self.assertEqual(4, len(objs))
+        self.assertEqual(4, len(set(objs)))
+
         for obj in objs:
             self.assertEqual(obj.id, 2)
             self.assertEqual(obj.foo, 'bar')
@@ -141,9 +162,14 @@ class SimpleBuildTestCase(unittest.TestCase):
         self.assertEqual(obj.id, None)
         self.assertEqual(obj.foo, 'bar')
 
-    @tools.disable_warnings
     def test_generate_create(self):
         obj = factory.generate(FakeModel, factory.CREATE_STRATEGY, foo='bar')
+        self.assertEqual(obj.id, None)
+        self.assertEqual(obj.foo, 'bar')
+
+    def test_generate_create_custom_base(self):
+        obj = factory.generate(FakeModel, factory.CREATE_STRATEGY, foo='bar',
+                FACTORY_CLASS=factory.DjangoModelFactory)
         self.assertEqual(obj.id, 2)
         self.assertEqual(obj.foo, 'bar')
 
@@ -162,13 +188,23 @@ class SimpleBuildTestCase(unittest.TestCase):
             self.assertEqual(obj.id, None)
             self.assertEqual(obj.foo, 'bar')
 
-    @tools.disable_warnings
     def test_generate_batch_create(self):
         objs = factory.generate_batch(FakeModel, factory.CREATE_STRATEGY, 20, foo='bar')
 
         self.assertEqual(20, len(objs))
         self.assertEqual(20, len(set(objs)))
 
+        for obj in objs:
+            self.assertEqual(obj.id, None)
+            self.assertEqual(obj.foo, 'bar')
+
+    def test_generate_batch_create_custom_base(self):
+        objs = factory.generate_batch(FakeModel, factory.CREATE_STRATEGY, 20, foo='bar',
+                FACTORY_CLASS=factory.DjangoModelFactory)
+
+        self.assertEqual(20, len(objs))
+        self.assertEqual(20, len(set(objs)))
+
         for obj in objs:
             self.assertEqual(obj.id, 2)
             self.assertEqual(obj.foo, 'bar')
@@ -188,9 +224,13 @@ class SimpleBuildTestCase(unittest.TestCase):
         self.assertEqual(obj.id, None)
         self.assertEqual(obj.foo, 'bar')
 
-    @tools.disable_warnings
     def test_simple_generate_create(self):
         obj = factory.simple_generate(FakeModel, True, foo='bar')
+        self.assertEqual(obj.id, None)
+        self.assertEqual(obj.foo, 'bar')
+
+    def test_simple_generate_create_custom_base(self):
+        obj = factory.simple_generate(FakeModel, True, foo='bar', FACTORY_CLASS=factory.DjangoModelFactory)
         self.assertEqual(obj.id, 2)
         self.assertEqual(obj.foo, 'bar')
 
@@ -204,13 +244,23 @@ class SimpleBuildTestCase(unittest.TestCase):
             self.assertEqual(obj.id, None)
             self.assertEqual(obj.foo, 'bar')
 
-    @tools.disable_warnings
     def test_simple_generate_batch_create(self):
         objs = factory.simple_generate_batch(FakeModel, True, 20, foo='bar')
 
         self.assertEqual(20, len(objs))
         self.assertEqual(20, len(set(objs)))
 
+        for obj in objs:
+            self.assertEqual(obj.id, None)
+            self.assertEqual(obj.foo, 'bar')
+
+    def test_simple_generate_batch_create_custom_base(self):
+        objs = factory.simple_generate_batch(FakeModel, True, 20, foo='bar',
+                FACTORY_CLASS=factory.DjangoModelFactory)
+
+        self.assertEqual(20, len(objs))
+        self.assertEqual(20, len(set(objs)))
+
         for obj in objs:
             self.assertEqual(obj.id, 2)
             self.assertEqual(obj.foo, 'bar')
@@ -232,7 +282,7 @@ class SimpleBuildTestCase(unittest.TestCase):
 
 
 class UsingFactoryTestCase(unittest.TestCase):
-    def testAttribute(self):
+    def test_attribute(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
@@ -262,12 +312,12 @@ class UsingFactoryTestCase(unittest.TestCase):
         test_object = InheritedFactory.build()
         self.assertEqual(test_object.one, 'one')
 
-    def testSequence(self):
+    def test_sequence(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
-            one = factory.Sequence(lambda n: 'one' + n)
-            two = factory.Sequence(lambda n: 'two' + n)
+            one = factory.Sequence(lambda n: 'one%d' % n)
+            two = factory.Sequence(lambda n: 'two%d' % n)
 
         test_object0 = TestObjectFactory.build()
         self.assertEqual(test_object0.one, 'one0')
@@ -277,7 +327,7 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual(test_object1.one, 'one1')
         self.assertEqual(test_object1.two, 'two1')
 
-    def testSequenceCustomBegin(self):
+    def test_sequence_custom_begin(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
@@ -285,8 +335,8 @@ class UsingFactoryTestCase(unittest.TestCase):
             def _setup_next_sequence(cls):
                 return 42
 
-            one = factory.Sequence(lambda n: 'one' + n)
-            two = factory.Sequence(lambda n: 'two' + n)
+            one = factory.Sequence(lambda n: 'one%d' % n)
+            two = factory.Sequence(lambda n: 'two%d' % n)
 
         test_object0 = TestObjectFactory.build()
         self.assertEqual('one42', test_object0.one)
@@ -296,6 +346,22 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual('one43', test_object1.one)
         self.assertEqual('two43', test_object1.two)
 
+    def test_sequence_override(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
+            one = factory.Sequence(lambda n: 'one%d' % n)
+
+        o1 = TestObjectFactory()
+        o2 = TestObjectFactory()
+        o3 = TestObjectFactory(__sequence=42)
+        o4 = TestObjectFactory()
+
+        self.assertEqual('one0', o1.one)
+        self.assertEqual('one1', o2.one)
+        self.assertEqual('one42', o3.one)
+        self.assertEqual('one2', o4.one)
+
     def test_custom_create(self):
         class TestModelFactory(factory.Factory):
             FACTORY_FOR = TestModel
@@ -333,8 +399,8 @@ class UsingFactoryTestCase(unittest.TestCase):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
-            one = factory.Sequence(lambda n: 'one' + n)
-            two = factory.Sequence(lambda n: 'two' + n)
+            one = factory.Sequence(lambda n: 'one%d' % n)
+            two = factory.Sequence(lambda n: 'two%d' % n)
 
         objs = TestObjectFactory.build_batch(20)
 
@@ -344,7 +410,7 @@ class UsingFactoryTestCase(unittest.TestCase):
             self.assertEqual('one%d' % i, obj.one)
             self.assertEqual('two%d' % i, obj.two)
 
-    def testLazyAttribute(self):
+    def test_lazy_attribute(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
@@ -355,12 +421,12 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual(test_object.one, 'abc')
         self.assertEqual(test_object.two, 'abc xyz')
 
-    def testLazyAttributeSequence(self):
+    def test_lazy_attribute_sequence(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
-            one = factory.LazyAttributeSequence(lambda a, n: 'abc' + n)
-            two = factory.LazyAttributeSequence(lambda a, n: a.one + ' xyz' + n)
+            one = factory.LazyAttributeSequence(lambda a, n: 'abc%d' % n)
+            two = factory.LazyAttributeSequence(lambda a, n: a.one + ' xyz%d' % n)
 
         test_object0 = TestObjectFactory.build()
         self.assertEqual(test_object0.one, 'abc0')
@@ -370,7 +436,7 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual(test_object1.one, 'abc1')
         self.assertEqual(test_object1.two, 'abc1 xyz1')
 
-    def testLazyAttributeDecorator(self):
+    def test_lazy_attribute_decorator(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
@@ -381,7 +447,7 @@ class UsingFactoryTestCase(unittest.TestCase):
         test_object = TestObjectFactory.build()
         self.assertEqual(test_object.one, 'one')
 
-    def testSelfAttribute(self):
+    def test_self_attribute(self):
         class TmpObj(object):
             n = 3
 
@@ -400,7 +466,7 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual(3, test_object.four)
         self.assertEqual(5, test_object.five)
 
-    def testSelfAttributeParent(self):
+    def test_self_attribute_parent(self):
         class TestModel2(FakeModel):
             pass
 
@@ -417,38 +483,38 @@ class UsingFactoryTestCase(unittest.TestCase):
         test_model = TestModel2Factory()
         self.assertEqual(4, test_model.two.three)
 
-    def testSequenceDecorator(self):
+    def test_sequence_decorator(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
             @factory.sequence
             def one(n):
-                return 'one' + n
+                return 'one%d' % n
 
         test_object = TestObjectFactory.build()
         self.assertEqual(test_object.one, 'one0')
 
-    def testLazyAttributeSequenceDecorator(self):
+    def test_lazy_attribute_sequence_decorator(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
             @factory.lazy_attribute_sequence
             def one(a, n):
-                return 'one' + n
+                return 'one%d' % n
             @factory.lazy_attribute_sequence
             def two(a, n):
-                return a.one + ' two' + n
+                return a.one + ' two%d' % n
 
         test_object = TestObjectFactory.build()
         self.assertEqual(test_object.one, 'one0')
         self.assertEqual(test_object.two, 'one0 two0')
 
-    def testBuildWithParameters(self):
+    def test_build_with_parameters(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
-            one = factory.Sequence(lambda n: 'one' + n)
-            two = factory.Sequence(lambda n: 'two' + n)
+            one = factory.Sequence(lambda n: 'one%d' % n)
+            two = factory.Sequence(lambda n: 'two%d' % n)
 
         test_object0 = TestObjectFactory.build(three='three')
         self.assertEqual(test_object0.one, 'one0')
@@ -459,7 +525,7 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual(test_object1.one, 'other')
         self.assertEqual(test_object1.two, 'two1')
 
-    def testCreate(self):
+    def test_create(self):
         class TestModelFactory(FakeModelFactory):
             FACTORY_FOR = TestModel
 
@@ -624,7 +690,7 @@ class UsingFactoryTestCase(unittest.TestCase):
             three = factory.Sequence(lambda n: int(n))
 
         objs = TestObjectFactory.stub_batch(20,
-            one=factory.Sequence(lambda n: n))
+            one=factory.Sequence(lambda n: str(n)))
 
         self.assertEqual(20, len(objs))
         self.assertEqual(20, len(set(objs)))
@@ -634,7 +700,7 @@ class UsingFactoryTestCase(unittest.TestCase):
             self.assertEqual('%d two' % i, obj.two)
             self.assertEqual(i, obj.three)
 
-    def testInheritance(self):
+    def test_inheritance(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
@@ -656,7 +722,7 @@ class UsingFactoryTestCase(unittest.TestCase):
         test_object_alt = TestObjectFactory.build()
         self.assertEqual(None, test_object_alt.three)
 
-    def testInheritanceWithInheritedClass(self):
+    def test_inheritance_with_inherited_class(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
@@ -673,7 +739,7 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual(test_object.three, 'three')
         self.assertEqual(test_object.four, 'three four')
 
-    def testDualInheritance(self):
+    def test_dual_inheritance(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
@@ -693,33 +759,7 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual('three', obj.three)
         self.assertEqual('four', obj.four)
 
-    @tools.disable_warnings
-    def test_set_building_function(self):
-        def building_function(class_to_create, **kwargs):
-            return "This doesn't even return an instance of {0}".format(class_to_create.__name__)
-
-        class TestModelFactory(FakeModelFactory):
-            FACTORY_FOR = TestModel
-
-        TestModelFactory.set_building_function(building_function)
-
-        test_object = TestModelFactory.build()
-        self.assertEqual(test_object, "This doesn't even return an instance of TestModel")
-
-    @tools.disable_warnings
-    def testSetCreationFunction(self):
-        def creation_function(class_to_create, **kwargs):
-            return "This doesn't even return an instance of {0}".format(class_to_create.__name__)
-
-        class TestModelFactory(FakeModelFactory):
-            FACTORY_FOR = TestModel
-
-        TestModelFactory.set_creation_function(creation_function)
-
-        test_object = TestModelFactory.create()
-        self.assertEqual(test_object, "This doesn't even return an instance of TestModel")
-
-    def testClassMethodAccessible(self):
+    def test_class_method_accessible(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
@@ -729,7 +769,7 @@ class UsingFactoryTestCase(unittest.TestCase):
 
         self.assertEqual(TestObjectFactory.alt_create(foo=1), {"foo": 1})
 
-    def testStaticMethodAccessible(self):
+    def test_static_method_accessible(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
@@ -739,6 +779,65 @@ class UsingFactoryTestCase(unittest.TestCase):
 
         self.assertEqual(TestObjectFactory.alt_create(foo=1), {"foo": 1})
 
+    def test_arg_parameters(self):
+        class TestObject(object):
+            def __init__(self, *args, **kwargs):
+                self.args = args
+                self.kwargs = kwargs
+
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            FACTORY_ARG_PARAMETERS = ('x', 'y')
+
+            x = 1
+            y = 2
+            z = 3
+            t = 4
+
+        obj = TestObjectFactory.build(x=42, z=5)
+        self.assertEqual((42, 2), obj.args)
+        self.assertEqual({'z': 5, 't': 4}, obj.kwargs)
+
+    def test_hidden_args(self):
+        class TestObject(object):
+            def __init__(self, *args, **kwargs):
+                self.args = args
+                self.kwargs = kwargs
+
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            FACTORY_HIDDEN_ARGS = ('x', 'z')
+
+            x = 1
+            y = 2
+            z = 3
+            t = 4
+
+        obj = TestObjectFactory.build(x=42, z=5)
+        self.assertEqual((), obj.args)
+        self.assertEqual({'y': 2, 't': 4}, obj.kwargs)
+
+    def test_hidden_args_and_arg_parameters(self):
+        class TestObject(object):
+            def __init__(self, *args, **kwargs):
+                self.args = args
+                self.kwargs = kwargs
+
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            FACTORY_HIDDEN_ARGS = ('x', 'z')
+            FACTORY_ARG_PARAMETERS = ('y',)
+
+            x = 1
+            y = 2
+            z = 3
+            t = 4
+
+        obj = TestObjectFactory.build(x=42, z=5)
+        self.assertEqual((2,), obj.args)
+        self.assertEqual({'t': 4}, obj.kwargs)
+
+
 
 class NonKwargParametersTestCase(unittest.TestCase):
     def test_build(self):
@@ -812,7 +911,7 @@ class KwargAdjustTestCase(unittest.TestCase):
 
 
 class SubFactoryTestCase(unittest.TestCase):
-    def testSubFactory(self):
+    def test_sub_factory(self):
         class TestModel2(FakeModel):
             pass
 
@@ -829,7 +928,7 @@ class SubFactoryTestCase(unittest.TestCase):
         self.assertEqual(1, test_model.id)
         self.assertEqual(1, test_model.two.id)
 
-    def testSubFactoryWithLazyFields(self):
+    def test_sub_factory_with_lazy_fields(self):
         class TestModel2(FakeModel):
             pass
 
@@ -839,7 +938,7 @@ class SubFactoryTestCase(unittest.TestCase):
         class TestModel2Factory(FakeModelFactory):
             FACTORY_FOR = TestModel2
             two = factory.SubFactory(TestModelFactory,
-                                          one=factory.Sequence(lambda n: 'x%sx' % n),
+                                          one=factory.Sequence(lambda n: 'x%dx' % n),
                                           two=factory.LazyAttribute(
                                               lambda o: '%s%s' % (o.one, o.one)))
 
@@ -847,7 +946,7 @@ class SubFactoryTestCase(unittest.TestCase):
         self.assertEqual('x0x', test_model.two.one)
         self.assertEqual('x0xx0x', test_model.two.two)
 
-    def testSubFactoryAndSequence(self):
+    def test_sub_factory_and_sequence(self):
         class TestObject(object):
             def __init__(self, **kwargs):
                 for k, v in kwargs.items():
@@ -868,7 +967,7 @@ class SubFactoryTestCase(unittest.TestCase):
         wrapping = WrappingTestObjectFactory.build()
         self.assertEqual(1, wrapping.wrapped.one)
 
-    def testSubFactoryOverriding(self):
+    def test_sub_factory_overriding(self):
         class TestObject(object):
             def __init__(self, **kwargs):
                 for k, v in kwargs.items():
@@ -895,7 +994,7 @@ class SubFactoryTestCase(unittest.TestCase):
         self.assertEqual(wrapping.wrapped.three, 3)
         self.assertEqual(wrapping.wrapped.four, 4)
 
-    def testNestedSubFactory(self):
+    def test_nested_sub_factory(self):
         """Test nested sub-factories."""
 
         class TestObject(object):
@@ -921,7 +1020,7 @@ class SubFactoryTestCase(unittest.TestCase):
         self.assertEqual(outer.wrap.wrapped.two, 2)
         self.assertEqual(outer.wrap.wrapped_bis.one, 1)
 
-    def testNestedSubFactoryWithOverriddenSubFactories(self):
+    def test_nested_sub_factory_with_overridden_sub_factories(self):
         """Test nested sub-factories, with attributes overridden with subfactories."""
 
         class TestObject(object):
@@ -949,7 +1048,7 @@ class SubFactoryTestCase(unittest.TestCase):
         self.assertEqual(outer.wrap.wrapped.two.four, 4)
         self.assertEqual(outer.wrap.friend, 5)
 
-    def testSubFactoryAndInheritance(self):
+    def test_sub_factory_and_inheritance(self):
         """Test inheriting from a factory with subfactories, overriding."""
         class TestObject(object):
             def __init__(self, **kwargs):
@@ -973,7 +1072,7 @@ class SubFactoryTestCase(unittest.TestCase):
         self.assertEqual(wrapping.wrapped.two, 4)
         self.assertEqual(wrapping.friend, 5)
 
-    def testDiamondSubFactory(self):
+    def test_diamond_sub_factory(self):
         """Tests the case where an object has two fields with a common field."""
         class InnerMost(object):
             def __init__(self, a, b):
@@ -1114,50 +1213,24 @@ class IteratorTestCase(unittest.TestCase):
         for i, obj in enumerate(objs):
             self.assertEqual(i + 10, obj.one)
 
-    def test_infinite_iterator_deprecated(self):
-        with warnings.catch_warnings(record=True) as w:
-            __warningregistry__.clear()
-
-            warnings.simplefilter('always')
-            class TestObjectFactory(factory.Factory):
-                FACTORY_FOR = TestObject
-
-                foo = factory.InfiniteIterator(range(5))
-
-            self.assertEqual(1, len(w))
-            self.assertIn('InfiniteIterator', str(w[0].message))
-            self.assertIn('deprecated', str(w[0].message))
-
-    @tools.disable_warnings
-    def test_infinite_iterator(self):
-        class TestObjectFactory(factory.Factory):
-            FACTORY_FOR = TestObject
-
-            one = factory.InfiniteIterator(range(5))
-
-        objs = TestObjectFactory.build_batch(20)
-
-        for i, obj in enumerate(objs):
-            self.assertEqual(i % 5, obj.one)
-
     @unittest.skipUnless(is_python2, "Scope bleeding fixed in Python3+")
     @tools.disable_warnings
-    def test_infinite_iterator_list_comprehension_scope_bleeding(self):
+    def test_iterator_list_comprehension_scope_bleeding(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
-            one = factory.InfiniteIterator([j * 3 for j in range(5)])
+            one = factory.Iterator([j * 3 for j in range(5)])
 
         # Scope bleeding: j will end up in TestObjectFactory's scope.
 
         self.assertRaises(TypeError, TestObjectFactory.build)
 
     @tools.disable_warnings
-    def test_infinite_iterator_list_comprehension_protected(self):
+    def test_iterator_list_comprehension_protected(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
 
-            one = factory.InfiniteIterator([_j * 3 for _j in range(5)])
+            one = factory.Iterator([_j * 3 for _j in range(5)])
 
         # Scope bleeding : _j will end up in TestObjectFactory's scope.
         # But factory_boy ignores it, as a protected variable.
@@ -1180,36 +1253,133 @@ class IteratorTestCase(unittest.TestCase):
         for i, obj in enumerate(objs):
             self.assertEqual(i + 10, obj.one)
 
-    def test_infinite_iterator_decorator_deprecated(self):
-        with warnings.catch_warnings(record=True) as w:
-            __warningregistry__.clear()
 
-            warnings.simplefilter('always')
-            class TestObjectFactory(factory.Factory):
-                FACTORY_FOR = TestObject
+class BetterFakeModelManager(object):
+    def __init__(self, keys, instance):
+        self.keys = keys
+        self.instance = instance
 
-                @factory.infinite_iterator
-                def one():
-                    return range(5)
+    def get_or_create(self, **kwargs):
+        defaults = kwargs.pop('defaults', {})
+        if kwargs == self.keys:
+            return self.instance, False
+        kwargs.update(defaults)
+        instance = FakeModel.create(**kwargs)
+        instance.id = 2
+        return instance, True
 
-            self.assertEqual(1, len(w))
-            self.assertIn('infinite_iterator', str(w[0].message))
-            self.assertIn('deprecated', str(w[0].message))
+    def values_list(self, *args, **kwargs):
+        return self
 
-    @tools.disable_warnings
-    def test_infinite_iterator_decorator(self):
-        class TestObjectFactory(factory.Factory):
-            FACTORY_FOR = TestObject
+    def order_by(self, *args, **kwargs):
+        return [1]
 
-            @factory.infinite_iterator
-            def one():
-                for i in range(5):
-                    yield i
 
-        objs = TestObjectFactory.build_batch(20)
+class BetterFakeModel(object):
+    @classmethod
+    def create(cls, **kwargs):
+        instance = cls(**kwargs)
+        instance.id = 1
+        return instance
 
-        for i, obj in enumerate(objs):
-            self.assertEqual(i % 5, obj.one)
+    def __init__(self, **kwargs):
+        for name, value in kwargs.items():
+            setattr(self, name, value)
+            self.id = None
+
+
+class DjangoModelFactoryTestCase(unittest.TestCase):
+    def test_simple(self):
+        class FakeModelFactory(factory.DjangoModelFactory):
+            FACTORY_FOR = FakeModel
+
+        obj = FakeModelFactory(one=1)
+        self.assertEqual(1, obj.one)
+        self.assertEqual(2, obj.id)
+
+    def test_existing_instance(self):
+        prev = BetterFakeModel.create(x=1, y=2, z=3)
+        prev.id = 42
+
+        class MyFakeModel(BetterFakeModel):
+            objects = BetterFakeModelManager({'x': 1}, prev)
+
+        class MyFakeModelFactory(factory.DjangoModelFactory):
+            FACTORY_FOR = MyFakeModel
+            FACTORY_DJANGO_GET_OR_CREATE = ('x',)
+            x = 1
+            y = 4
+            z = 6
+
+        obj = MyFakeModelFactory()
+        self.assertEqual(prev, obj)
+        self.assertEqual(1, obj.x)
+        self.assertEqual(2, obj.y)
+        self.assertEqual(3, obj.z)
+        self.assertEqual(42, obj.id)
+
+    def test_existing_instance_complex_key(self):
+        prev = BetterFakeModel.create(x=1, y=2, z=3)
+        prev.id = 42
+
+        class MyFakeModel(BetterFakeModel):
+            objects = BetterFakeModelManager({'x': 1, 'y': 2, 'z': 3}, prev)
+
+        class MyFakeModelFactory(factory.DjangoModelFactory):
+            FACTORY_FOR = MyFakeModel
+            FACTORY_DJANGO_GET_OR_CREATE = ('x', 'y', 'z')
+            x = 1
+            y = 4
+            z = 6
+
+        obj = MyFakeModelFactory(y=2, z=3)
+        self.assertEqual(prev, obj)
+        self.assertEqual(1, obj.x)
+        self.assertEqual(2, obj.y)
+        self.assertEqual(3, obj.z)
+        self.assertEqual(42, obj.id)
+
+    def test_new_instance(self):
+        prev = BetterFakeModel.create(x=1, y=2, z=3)
+        prev.id = 42
+
+        class MyFakeModel(BetterFakeModel):
+            objects = BetterFakeModelManager({'x': 1}, prev)
+
+        class MyFakeModelFactory(factory.DjangoModelFactory):
+            FACTORY_FOR = MyFakeModel
+            FACTORY_DJANGO_GET_OR_CREATE = ('x',)
+            x = 1
+            y = 4
+            z = 6
+
+        obj = MyFakeModelFactory(x=2)
+        self.assertNotEqual(prev, obj)
+        self.assertEqual(2, obj.x)
+        self.assertEqual(4, obj.y)
+        self.assertEqual(6, obj.z)
+        self.assertEqual(2, obj.id)
+
+    def test_new_instance_complex_key(self):
+        prev = BetterFakeModel.create(x=1, y=2, z=3)
+        prev.id = 42
+
+        class MyFakeModel(BetterFakeModel):
+            objects = BetterFakeModelManager({'x': 1, 'y': 2, 'z': 3}, prev)
+
+        class MyFakeModelFactory(factory.DjangoModelFactory):
+            FACTORY_FOR = MyFakeModel
+            FACTORY_DJANGO_GET_OR_CREATE = ('x', 'y', 'z')
+            x = 1
+            y = 4
+            z = 6
+
+        obj = MyFakeModelFactory(y=2, z=4)
+        self.assertNotEqual(prev, obj)
+        self.assertEqual(1, obj.x)
+        self.assertEqual(2, obj.y)
+        self.assertEqual(4, obj.z)
+        self.assertEqual(2, obj.id)
 
 
 class PostGenerationTestCase(unittest.TestCase):
@@ -1252,25 +1422,6 @@ class PostGenerationTestCase(unittest.TestCase):
         self.assertFalse(obj.create)
         self.assertEqual({'incr_one': 42}, obj.results)
 
-    @tools.disable_warnings
-    def test_post_generation_calling(self):
-        class TestObjectFactory(factory.Factory):
-            FACTORY_FOR = TestObject
-
-            one = 1
-
-            @factory.post_generation()
-            def incr_one(self, _create, _increment):
-                self.one += 1
-
-        obj = TestObjectFactory.build()
-        self.assertEqual(2, obj.one)
-        self.assertFalse(hasattr(obj, 'incr_one'))
-
-        obj = TestObjectFactory.build(one=2)
-        self.assertEqual(3, obj.one)
-        self.assertFalse(hasattr(obj, 'incr_one'))
-
     def test_post_generation_extraction(self):
         class TestObjectFactory(factory.Factory):
             FACTORY_FOR = TestObject
@@ -1440,6 +1591,172 @@ class CircularTestCase(unittest.TestCase):
         self.assertIsNone(b.foo.bar.foo.bar)
 
 
+class DictTestCase(unittest.TestCase):
+    def test_empty_dict(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = factory.Dict({})
+
+        o = TestObjectFactory()
+        self.assertEqual({}, o.one)
+
+    def test_naive_dict(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = factory.Dict({'a': 1})
+
+        o = TestObjectFactory()
+        self.assertEqual({'a': 1}, o.one)
+
+    def test_sequence_dict(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = factory.Dict({'a': factory.Sequence(lambda n: n + 2)})
+
+        o1 = TestObjectFactory()
+        o2 = TestObjectFactory()
+
+        self.assertEqual({'a': 2}, o1.one)
+        self.assertEqual({'a': 3}, o2.one)
+
+    def test_dict_override(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = factory.Dict({'a': 1})
+
+        o = TestObjectFactory(one__a=2)
+        self.assertEqual({'a': 2}, o.one)
+
+    def test_dict_extra_key(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = factory.Dict({'a': 1})
+
+        o = TestObjectFactory(one__b=2)
+        self.assertEqual({'a': 1, 'b': 2}, o.one)
+
+    def test_dict_merged_fields(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            two = 13
+            one = factory.Dict({
+                'one': 1,
+                'two': 2,
+                'three': factory.SelfAttribute('two'),
+            })
+
+        o = TestObjectFactory(one__one=42)
+        self.assertEqual({'one': 42, 'two': 2, 'three': 2}, o.one)
+
+    def test_nested_dicts(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = 1
+            two = factory.Dict({
+                'one': 3,
+                'two': factory.SelfAttribute('one'),
+                'three': factory.Dict({
+                    'one': 5,
+                    'two': factory.SelfAttribute('..one'),
+                    'three': factory.SelfAttribute('...one'),
+                }),
+            })
+
+        o = TestObjectFactory()
+        self.assertEqual(1, o.one)
+        self.assertEqual({
+            'one': 3,
+            'two': 3,
+            'three': {
+                'one': 5,
+                'two': 3,
+                'three': 1,
+            },
+        }, o.two)
+
+
+class ListTestCase(unittest.TestCase):
+    def test_empty_list(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = factory.List([])
+
+        o = TestObjectFactory()
+        self.assertEqual([], o.one)
+
+    def test_naive_list(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = factory.List([1])
+
+        o = TestObjectFactory()
+        self.assertEqual([1], o.one)
+
+    def test_sequence_list(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = factory.List([factory.Sequence(lambda n: n + 2)])
+
+        o1 = TestObjectFactory()
+        o2 = TestObjectFactory()
+
+        self.assertEqual([2], o1.one)
+        self.assertEqual([3], o2.one)
+
+    def test_list_override(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = factory.List([1])
+
+        o = TestObjectFactory(one__0=2)
+        self.assertEqual([2], o.one)
+
+    def test_list_extra_key(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = factory.List([1])
+
+        o = TestObjectFactory(one__1=2)
+        self.assertEqual([1, 2], o.one)
+
+    def test_list_merged_fields(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            two = 13
+            one = factory.List([
+                1,
+                2,
+                factory.SelfAttribute('1'),
+            ])
+
+        o = TestObjectFactory(one__0=42)
+        self.assertEqual([42, 2, 2], o.one)
+
+    def test_nested_lists(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = 1
+            two = factory.List([
+                3,
+                factory.SelfAttribute('0'),
+                factory.List([
+                    5,
+                    factory.SelfAttribute('..0'),
+                    factory.SelfAttribute('...one'),
+                ]),
+            ])
+
+        o = TestObjectFactory()
+        self.assertEqual(1, o.one)
+        self.assertEqual([
+            3,
+            3,
+            [
+                5,
+                3,
+                1,
+            ],
+        ], o.two)
 
 if __name__ == '__main__':
     unittest.main()
